#include <iostream>
#include <fstream>
#include "classes.h"
#include "mfgen.h"
#include "mfgen.rh"

//---------------------------------------------------------------------------
const char* header=	"#Make file generated by  mfgen.exe\n" \
										"#Michel Leunen http://www.leunen.com/\n\n" \
										"BPATH=$(MAKEDIR)\n" \
										"INCLUDEPATH=$(BPATH)\\..\\include\n" \
										"LIBPATH=$(BPATH)\\..\\lib\n\n" \
										"CC=$(BPATH)\\bcc32\n" \
										"RC=$(BPATH)\\brcc32\n";
const char* cflags= "-H- -I$(INCLUDEPATH)";
const char* lflags= "-x -Gn -L$(LIBPATH)";
const char* rflags= "-32 -i$(INCLUDEPATH)";
const char* libs=		"import32.lib";
const char* link=		"\n\n.autodepend\n\n" \
										"all: $(PROJECT)\n\n" \
										"$(PROJECT): $(OBJS) $(RES)\n" \
										"\t$(LINK) $(LFLAGS) $(OBJS) $(SYSOBJS),$(PROJECT),,$(SYSLIBS) $(LIBS),$(DEFS),$(RES)\n\n";
const char* liblink=
										"\n\n.autodepend\n\n" \
										"all: $(PROJECT)\n\n" \
										"$(PROJECT): $(OBJS) $(RES)\n" \
										"\t$(LINK) /u $(PROJECT) +$(OBJS)\n\n";
const char* dependencies=
										".cpp.obj:\n" \
										"\t$(CC) -c $(CFLAGS) $<\n\n" \
										".cc.obj:\n" \
										"\t$(CC) -c -P $(CFLAGS) $<\n\n" \
										".c.obj:\n" \
										"\t$(CC) -c $(CFLAGS) $<\n\n" \
										".rc.res:\n" \
										"\t$(RC) $(RFLAGS) $<\n";
										
//---------------------------------------------------------------------------
MakeFileOptions::MakeFileOptions(HWND owner):owner_(owner)
{
	//Get state of radio buttons and check boxes
	if(GetCheckState(owner,ID_RB_CONSOLE))target_=tConsole;
	else if(GetCheckState(owner,ID_RB_WINDOWS))target_=tWindows;
		else if(GetCheckState(owner,ID_RB_DLL))target_=tDll;
			else if(GetCheckState(owner,ID_RB_LIB))target_=tLib;
	if(GetCheckState(owner,ID_CB_MULTITHREAD))threaded_=true;
	else threaded_=false;
	if(GetCheckState(owner,ID_CB_DEBUG))debug_=true;
	else debug_=false;
	//Get the files to compile and parse them
	for(int i=0;i<GetListboxCount(owner_,ID_LB_FILES);++i)
	{
		char file[MAX_PATH];
		GetListboxItemText(owner_,ID_LB_FILES,i,file);
		char ext[4];
		GetFileExtension(file,ext);
		ToLower(ext);
		if(!strcmp(ext,"cpp") || !strcmp(ext,"cc") ||	!strcmp(ext,"c"))
		{
			//Do not put a space in front of the file if it's 
			//the first 
			if(!cppfiles_.empty())cppfiles_.append(" ");
			//Replace the extension .cpp with .obj and add it to the list
			cppfiles_.append(ReplaceExt(file,".obj"));
		}
		if(strcmp(ext,"lib") == 0)
		{
			if(!libfiles_.empty())libfiles_.append(" ");
			libfiles_.append(file);
		}
		if(strcmp(ext,"rc") == 0)
		{
			if(!rcfiles_.empty())rcfiles_.append(" ");
			rcfiles_.append(ReplaceExt(file,".res"));
		}
		if(strcmp(ext,"obj") == 0)
		{
			if(!objfiles_.empty())objfiles_.append(" ");
			objfiles_.append(file);
		}
		if(strcmp(ext,"def") == 0)
		{
			if(!deffiles_.empty())deffiles_.append(" ");
			deffiles_.append(file);
		}
	}
}
//---------------------------------------------------------------------------
std::string MakeFileOptions::ReplaceExt(const std::string& filename,const std::string& ext)
{
	std::string fn=filename;
	std::string::size_type pos=fn.find_last_of(".");
	if(pos==std::string::npos)return fn.append(ext);
	return fn.substr(0,pos).append(ext);
}
//---------------------------------------------------------------------------
bool MakeFileGenerator::GenerateConsoleMakeFile(const std::string &cppfiles,
                                                const std::string &libfiles,
				                                        const std::string &rcfiles,
				                                        const std::string &objfiles,
				                                        const std::string &deffiles,
				                                        bool threaded,
				                                        bool debug)
{
	std::string mf;
	mf.append(header);
	mf.append("LINK=$(BPATH)\\ilink32\n\n");
	mf.append("SYSOBJS=c0x32.obj\n");
	if(threaded)mf.append("SYSLIBS=cw32mt.lib ");
	else mf.append("SYSLIBS=cw32.lib ");
	mf.append(libs);
	mf.append("\nCFLAGS=-tWC ");
	if(debug)mf.append("-Od -v -vi- -y ");
	else mf.append("-O2 ");
	if(threaded)mf.append("-tWM ");
	else mf.append("-tWM- ");
	mf.append(cflags);
	mf.append("\nLFLAGS=-Tpe -ap ");
	mf.append(lflags);
	mf.append("\nRFLAGS=");
	mf.append(rflags);
	mf.append("\n\nPROJECT=");
	mf.append(ChangeExt(makefilename_,".exe"));
	mf.append("\nOBJS=");
	mf.append(cppfiles);
	if(!objfiles.empty())
	{
		mf.append(" ");
		mf.append(objfiles);
	}
	mf.append("\nRES=");
	mf.append(rcfiles);
	mf.append("\nLIBS=");
	mf.append(libfiles);
	mf.append("\nDEFS=");
	mf.append(deffiles);
	mf.append(link);
	mf.append(dependencies);
	if(SaveMakefile(mf))return true;
	else return false;	
}
//---------------------------------------------------------------------------
bool MakeFileGenerator::GenerateWindowsMakeFile(const std::string &cppfiles,
                                                const std::string &libfiles,
				                                        const std::string &rcfiles,
				                                        const std::string &objfiles,
				                                        const std::string &deffiles,
				                                        bool threaded,
				                                        bool debug)
{
	std::string mf;
	mf.append(header);
	mf.append("LINK=$(BPATH)\\ilink32\n\n");	
	mf.append("SYSOBJS=c0w32.obj\n");
	if(threaded)mf.append("SYSLIBS=cw32mt.lib ");
	else mf.append("SYSLIBS=cw32.lib ");
	mf.append(libs);
	mf.append("\nCFLAGS=-tW ");
	if(debug)mf.append("-Od -v -vi- -y ");
	else mf.append("-O2 ");
	if(threaded)mf.append("-tWM ");
	else mf.append("-tWM- ");
	mf.append(cflags);
	mf.append("\nLFLAGS=-Tpe -aa ");
	mf.append(lflags);
	mf.append("\nRFLAGS=");
	mf.append(rflags);
	mf.append("\n\nPROJECT=");
	mf.append(ChangeExt(makefilename_,".exe"));
	mf.append("\nOBJS=");
	mf.append(cppfiles);
	if(!objfiles.empty())
	{
		mf.append(" ");
		mf.append(objfiles);
	}
	mf.append("\nRES=");
	mf.append(rcfiles);
	mf.append("\nLIBS=");
	mf.append(libfiles);
	mf.append("\nDEFS=");
	mf.append(deffiles);
	mf.append(link);
	mf.append(dependencies);	
	if(SaveMakefile(mf))return true;
	else return false;	
}
//---------------------------------------------------------------------------
bool MakeFileGenerator::GenerateDllMakeFile(const std::string &cppfiles,
                                            const std::string &libfiles,
				                                    const std::string &rcfiles,
				                                    const std::string &objfiles,
				                                    const std::string &deffiles,
				                                    bool threaded,
				                                    bool debug)
{
	std::string mf;
	mf.append(header);
	mf.append("LINK=$(BPATH)\\ilink32\n\n");
	mf.append("SYSOBJS=c0d32.obj\n");
	if(threaded)mf.append("SYSLIBS=cw32mt.lib ");
	else mf.append("SYSLIBS=cw32.lib ");
	mf.append(libs);
	mf.append("\nCFLAGS=-tWD ");
	if(debug)mf.append("-Od -v -vi- -y ");
	else mf.append("-O2 ");
	if(threaded)mf.append("-tWM ");
	else mf.append("-tWM- ");
	mf.append(cflags);
	mf.append("\nLFLAGS=-Tpd -aa ");
	mf.append(lflags);
	mf.append("\nRFLAGS=");
	mf.append(rflags);
	mf.append("\n\nPROJECT=");
	mf.append(ChangeExt(makefilename_,".dll"));
	mf.append("\nOBJS=");
	mf.append(cppfiles);
	if(!objfiles.empty())
	{
		mf.append(" ");
		mf.append(objfiles);
	}
	mf.append("\nRES=");
	mf.append(rcfiles);
	mf.append("\nLIBS=");
	mf.append(libfiles);
	mf.append("\nDEFS=");
	mf.append(deffiles);
	mf.append(link);
	mf.append(dependencies);
	if(SaveMakefile(mf))return true;
	else return false;	
}
//---------------------------------------------------------------------------
bool MakeFileGenerator::GenerateLibMakeFile(const std::string &cppfiles,
                                            const std::string &libfiles,
				                                    const std::string &rcfiles,
				                                    const std::string &objfiles,
				                                    const std::string &deffiles,
				                                    bool threaded,
				                                    bool debug)
{
	std::string mf;
	mf.append(header);
	mf.append("LINK=$(BPATH)\\tlib\n\n");
	mf.append("SYSOBJS=\n");
	mf.append("SYSLIBS=\n");
	mf.append("CFLAGS=-tW ");
	if(debug)mf.append("-Od -v -vi- -y ");
	else mf.append("-O2 ");
	if(threaded)mf.append("-tWM ");
	else mf.append("-tWM- ");
	mf.append(cflags);
	mf.append("\nLFLAGS=");
	mf.append("\nRFLAGS=");
	mf.append(rflags);
	mf.append("\n\nPROJECT=");
	mf.append(ChangeExt(makefilename_,".lib"));
	mf.append("\nOBJS=");
	mf.append(cppfiles);
	if(!objfiles.empty())
	{
		mf.append(" ");
		mf.append(objfiles);
	}
	mf.append("\nRES=");
	mf.append(rcfiles);
	mf.append("\nLIBS=");
	mf.append(libfiles);
	mf.append("\nDEFS=");
	mf.append(deffiles);
	mf.append(liblink);
	mf.append(dependencies);
	if(SaveMakefile(mf))return true;
	else return false;	
}
//---------------------------------------------------------------------------
std::string MakeFileGenerator::GetPath(const std::string& filename)
{
	std::string fn=filename;
	std::string::size_type pos=fn.find_last_of("\\");
	if(pos==std::string::npos)return "";
	return fn.substr(0,pos-1);
}
//---------------------------------------------------------------------------
bool MakeFileGenerator::SaveMakefile(const std::string& str)
{
	std::ofstream out(makefilename_.c_str(),std::ios::trunc);
  if(!out)return false;
  out<<str;
  return true;
}
//---------------------------------------------------------------------------
std::string MakeFileGenerator::ChangeExt(const std::string str,const std::string ext)
{
	std::string fn=str;
	std::string::size_type pos=str.find_last_of("\\");
	if(pos==std::string::npos)fn=str;
	else fn=str.substr(pos+1,str.length()-pos);
	pos=fn.find_last_of(".");
	if(pos==std::string::npos)fn.append(ext);
	else
	{
		fn=fn.substr(0,pos);
		fn.append(ext);
	}
	return fn;
}
//---------------------------------------------------------------------------
