/* mfgen.cpp 1.1
 * Michel Leunen 2003 http://www.leunen.com/cbuilder/
 * Generate a makefile for the free commandline tool BCC5.5.1 from Borland.
 * This program is free and comes with all source files. Use it at your own
 * risk.
 * Please report any bug or modification to michel@leunen.com
 *
 * v1.1 Added a -P switch when compiling .cc files assuming these are
 *      cpp files.
 *      Added the possibility to include a .def file to the project
 */

#include <windows.h>
#include <commctrl.h>
#include "mfgen.h"
#include "mfgen.rh"
#include "classes.h"

extern HINSTANCE _hInstance; //created by the startup code
//---------------------------------------------------------------------------
#pragma argsused
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
{
	//Is there a way to get it more concise?
	return DialogBox(_hInstance,MAKEINTRESOURCE(DLG_MAIN),NULL,DlgProc);
}
//---------------------------------------------------------------------------

void ShowHelp(HWND owner)
{
	MessageBox(owner,"MakeFile Generator v1.1:\n\n" \
						"Select the files to compile (.cpp,.c,.cc,.lib,.rc,.obj,.def).\n" \
						"All files must be in the same directory.\n" \
						"Hit 'Generate', give a name to your makefile and save it in the same\n" \
						"directory where your .cpp files are.\n\n" \
						"A special thanks to Harold Howe and Ed Mulroy (TeamB\n" \
						"members) for their help in the Borland commandlinetools newsgroup.\n" \
						"This program was build using the free commandline tool Bcc5.5.1.\n\n" \
						"Copyright  2003 Michel Leunen http://www.leunen.com/cbuilder/",
						"Help",MB_OK | MB_ICONQUESTION);
}
//---------------------------------------------------------------------------
//Insert a string in the listbox
int AddListboxItem(HWND owner,int id,const char* str)
{
	//Hack to get the horizontal scrollbar for the listbox
	static LONG gtLength=0;
	HDC lbDC=GetDC(GetDlgItem(owner,id));
	SIZE sz;
	HFONT hfont=(HFONT)SelectObject(lbDC,(HFONT*)SendDlgItemMessage(owner,id,WM_GETFONT,0,0));
	GetTextExtentPoint32(lbDC,str,strlen(str),&sz);
	if(sz.cx>gtLength)gtLength=sz.cx;
	SelectObject(lbDC,hfont);
	ReleaseDC(owner,lbDC);
	SendDlgItemMessage(owner,id,LB_SETHORIZONTALEXTENT,(WPARAM)gtLength+5,0);
	return SendDlgItemMessage(owner,id,LB_ADDSTRING,0,(LPARAM)str);
}
//---------------------------------------------------------------------------
//Get the number of items in a listbox
int GetListboxCount(HWND owner,int id)
{
	HWND hlist=GetDlgItem(owner,id);
	return SendMessage(hlist,LB_GETCOUNT,0,0);
}
//---------------------------------------------------------------------------
//Clear the content of a listbox
void ClearListBox(HWND owner,int id)
{
	SendDlgItemMessage(owner,id,LB_RESETCONTENT,0,0);
}
//---------------------------------------------------------------------------
//Get the selected item text from a listbox
int GetListboxSelItemIndex(HWND owner,int id)
{
	return SendDlgItemMessage(owner,id,LB_GETCURSEL,0,0);
}
//---------------------------------------------------------------------------
//Get the listbox item text with the specified index
//The value returned is the string length
int GetListboxItemText(HWND owner,int id,int index,char* buf)
{
	return SendDlgItemMessage(owner,id,LB_GETTEXT,(WPARAM)index,(LPARAM)buf);
}
//---------------------------------------------------------------------------
//Remove an item from a listbox
//The return value is the number of items remaining.
int RemoveListboxItem(HWND owner,int id,int index)
{
	return SendDlgItemMessage(owner,id,LB_DELETESTRING,(WPARAM)index,0);
}
//---------------------------------------------------------------------------
//Open the file open dialogbox and return the filename selected
bool OpenFileName(HWND hwnd,char* filename)
{
	OPENFILENAME ofn;
	ZeroMemory(&ofn,sizeof(OPENFILENAME));
	ofn.lStructSize=sizeof(OPENFILENAME);//OPENFILENAME_SIZE_VERSION_400;
	ofn.hwndOwner=hwnd;
	ofn.lpstrFilter="Cpp files (*.cpp)\0*.cpp\0Cpp files (*.cc)\0*.cc\0C files (*.c)\0*.c\0Resource files (*.rc)\0" \
									"*.rc\0Library files (*.lib)\0*.lib\0Obj files (*.obj)\0*.def\0Def files (*.def)\0*.obj\0" \
									"All files (*.*)\0*.*\0";
	ofn.lpstrFile=filename;
	ofn.nMaxFile=MAX_PATH;
	ofn.lpstrDefExt="cpp";
	if(GetOpenFileName(&ofn))return true;
	else return false;
}
//---------------------------------------------------------------------------
//Open the save dialog box and return a filename
bool SaveFileName(HWND hwnd,char* filename)
{
	OPENFILENAME ofn;
	ZeroMemory(&ofn,sizeof(OPENFILENAME));
	ofn.lStructSize=sizeof(OPENFILENAME);//OPENFILENAME_SIZE_VERSION_400;
	ofn.hwndOwner=hwnd;
	ofn.lpstrFilter="Make files (*.mak)\0*.mak\0All files (*.*)\0*.*\0";
	ofn.lpstrTitle="Save Project Makefile";
	ofn.nMaxFile=MAX_PATH;
	ofn.lpstrDefExt="mak";
	ofn.lpstrFile=filename;
	ofn.Flags=OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
	if(GetSaveFileName(&ofn))return true;
	else return false;
}
//---------------------------------------------------------------------------
//Returns the state of a radio button or a check box.
bool GetCheckState(HWND owner,int id)
{
	return SendDlgItemMessage(owner,id,BM_GETCHECK,0,0);	
}
//---------------------------------------------------------------------------
//Extract the file extension from a filename
void GetFileExtension(char* filename,char* ext)
{
	char *p=filename+strlen(filename);
	while(*p!='.' && p!=filename)--p;
	strcpy(ext,++p);
}
//---------------------------------------------------------------------------
//Extract the filename from a complete path+filename
void ExtractFilenameFromPath(char* path,char* filename)
{
	char *p=path+strlen(path);
	while(*p!='\\' && p!=path)--p;
	strcpy(filename,++p);
}
//---------------------------------------------------------------------------
//Transform uppercase string to lowercase
void ToLower(char* str)
{
  size_t len = strlen(str);
  for(size_t i=0;i<len;++i)str[i] = tolower(str[i]);
}
//---------------------------------------------------------------------------
//Dialog procedure
#pragma argsused
BOOL CALLBACK DlgProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch(msg)
	{
		case WM_CLOSE:
			EndDialog(hwnd,0);
			break;
			
		case WM_INITDIALOG:
			//default=windows application
			SendDlgItemMessage(hwnd,ID_RB_WINDOWS,BM_SETCHECK,BST_CHECKED,0);
			//default=debug mode
			SendDlgItemMessage(hwnd,ID_CB_DEBUG,BM_SETCHECK,BST_CHECKED,0);
			//Set dialog icon
			SendMessage(hwnd,WM_SETICON,ICON_SMALL,(LPARAM)LoadIcon(GetModuleHandle(NULL),
					MAKEINTRESOURCE(ID_ICON_BIG)));
			SendMessage(hwnd,WM_SETICON,ICON_BIG,(LPARAM)LoadIcon(GetModuleHandle(NULL),
					MAKEINTRESOURCE(ID_ICON_BIG)));
			break;
			
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case ID_BUT_ADD:
				{
					char FileName[MAX_PATH]="";
					if(OpenFileName(hwnd,FileName))
					{
						char buf[MAX_PATH];
						ExtractFilenameFromPath(FileName,buf);
						AddListboxItem(hwnd,ID_LB_FILES,buf);
					}
					break;
				}
					
				case ID_BUT_REMOVE:
				{
					unsigned int index=GetListboxSelItemIndex(hwnd,ID_LB_FILES);
					if(index!=LB_ERR)RemoveListboxItem(hwnd,ID_LB_FILES,index);
					break;
				}
					
				case ID_BUT_GENMAKE:
				{
					if(GetListboxCount(hwnd,ID_LB_FILES)==0)
					{
						MessageBox(hwnd,"Select the files to include in your project.",
								"WARNING: No project files",MB_ICONEXCLAMATION | MB_OK);
						break;
					}
					char FileName[MAX_PATH]="";
					if(SaveFileName(hwnd,FileName))
					{
						//set cursor to hourglass
						HCURSOR hDefaultCur=GetCursor();
						HCURSOR hHourglassCur = LoadCursor(NULL, IDC_WAIT);
						SetCursor(hHourglassCur);
						//Get makefile options
						MakeFileOptions mo(hwnd);
						//Generate makefile
						MakeFileGenerator mfg(FileName);
						Target target=mo.GetTarget();
						switch(target)
						{
							case tConsole:
								if(!mfg.GenerateConsoleMakeFile(mo.GetCppFiles(),
								                                mo.GetLibFiles(),
								                                mo.GetRCFiles(),
										                            mo.GetObjFiles(),
										                            mo.GetDefFiles(),
										                            mo.Threaded(),
										                            mo.Debug()))
								{
									MessageBox(hwnd,"Can't generate the makefile.",
										"ERROR",MB_ICONERROR | MB_OK);
								}
								break;
							case tWindows:
								if(!mfg.GenerateWindowsMakeFile(mo.GetCppFiles(),
								                                mo.GetLibFiles(),
								                                mo.GetRCFiles(),
										                            mo.GetObjFiles(),
										                            mo.GetDefFiles(),
										                            mo.Threaded(),
										                            mo.Debug()))
								{
									MessageBox(hwnd,"Can't generate the makefile.",
										"ERROR",MB_ICONERROR | MB_OK);
								}
								break;
							case tDll:
								if(!mfg.GenerateDllMakeFile(mo.GetCppFiles(),
								                            mo.GetLibFiles(),
								                            mo.GetRCFiles(),
										                        mo.GetObjFiles(),
										                        mo.GetDefFiles(),
										                        mo.Threaded(),
										                        mo.Debug()))
								{
									MessageBox(hwnd,"Can't generate the makefile.",
										"ERROR",MB_ICONERROR | MB_OK);
								}
								break;
							case tLib:
								if(!mfg.GenerateLibMakeFile(mo.GetCppFiles(),
								                            mo.GetLibFiles(),
								                            mo.GetRCFiles(),
										                        mo.GetObjFiles(),
										                        mo.GetDefFiles(),
										                        mo.Threaded(),
										                        mo.Debug()))
								{
									MessageBox(hwnd,"Can't generate the makefile.",
										"ERROR",MB_ICONERROR | MB_OK);
								}
								break;
						}
						//restore original cursor
						SetCursor(hDefaultCur);
						char buf[MAX_PATH];
						ExtractFilenameFromPath(FileName,buf);
						strcat(buf," - Done!");
						SetWindowText(hwnd,buf);
					}
					break;
				}
					
				case ID_BUT_HELP:
					ShowHelp(hwnd);
					break;
			}
		default:
			return FALSE;
	}
  return TRUE;
}
